package wtalk;

/*
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */


/**
 * Interface wtalk.WTalkProtocol uses to manipulate a user list in the shell.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
public interface IUserList{
	/**
	 * Adds a connection named <em>connectionName</em>.  
	 * If the connection already exists, no change is made.
	 * <p>
	 * If there is already a connection by this name, 
	 * 	there is no change made.
	 *
	 * @param connectionName The name of connection to be added.
	 */
	public void addConnection(String connectionName);

	/**
	 * Adds a group named <em>groupName</em> to <em>connectionName</em>.
	 * If <em>groupName</em> already exists 
	 *	under <em>connectionName</em>, no change is made.
	 * If <em>connectionName</em> does not exist, no change is made.
	 * <p>
	 * If the specified group already exists, no change is made.
	 *
	 * @param groupName The group to be added.
	 * @param connectionName An existing connection to which group 
	 *	is to be added.
	 */
	public void addGroup(String groupName, String connectionName);

	/**
	 * Adds <em>userName</em> to the specified group.
	 * If the user already exists in the group, 
	 * 	or the group does not exist under the given connection,
	 *	no change is made.
	 * <p>
	 * If the specified user already exists, no change is made.
	 *
	 * @param userName The name of the user to add.
	 * @param groupName The name of the group to add <em>userName</em> to.
	 * @param connectionName The connection in which to search for
	 *	groupName.
	 */
	public void addUser(String userName, String groupName, String connectionName);


	/**
	 * Removes the named connection, and everything under it.
	 * No change is made if <em>connectionName</em> does not exist.
	 *
	 * @param connectionName Name of connection to remove.
	 */
	public void delConnection(String connectionName);

	/**
	 * Removes the named group, and all users under it.
	 * No change is made if either the group or connection named
	 * 	do not exist.
	 *
	 * @param groupName Name of group to be removed.
	 * @param connectionName Name of connection under which group exists.
	 */
	public void delGroup(String groupName, String connectionName);

	/**
	 * Removes the named user, assuming user exists under the named
	 * 	group and connection. There should be no change otherwise.
	 *
	 * @param userName User to be removed from the user list.
	 * @param groupName Group name to which User belongs.
	 * @param connectionName Connection under which to search 
	 *	for <em>groupName</em>.
	 */
	public void delUser(String userName, String groupName, String connectionName);
}
