package wtalk.protocols.irc;

import java.io.*;
import java.net.*;


import wtalk.RMsgHandler;

class ReceivedListener extends Thread{
	private IRC parent=null;
	private BufferedReader in=null;
	private IRCProtocol mIRC=null;
	private RMsgHandler rmh=null;	// generic message handler
	private RMsgHandler rwh = null;	// handles wall messages if !null
	private RMsgHandler rsh = null;	// handles system messages if !null
	private RMsgHandler rph = null;	// handles private messages if !null
	private RMsgHandler rah = null;	// handles automated messages if !null
	private StatusListener sl = null; //listens for status resposnes

	public ReceivedListener(IRC par){
		parent = par;
		in = null; rmh = null; mIRC=null;
	}

	public void ListenTo(BufferedReader n){
		in = n;
	}

	public void ReportTo(RMsgHandler r){
		rmh = r;
	}

	public void ReportWallTo(RMsgHandler r){
		rwh = r;
	}

	public void ReportSystemTo(RMsgHandler r){
		rsh = r;
	}

	public void ReportPrivateTo(RMsgHandler r){
		rph = r;
	}

	public void ReportAutoTo(RMsgHandler r){
		rah = r;
	}

	public void setStatusListener(StatusListener s){
		sl = s;
	}

	public void setIRCProtocol(IRCProtocol irc) { mIRC = irc; };


	public void CTCPHandle(String command, String target, 
			String from, String data, boolean bOriginator,
			boolean bPrivMsg) {


		// Is it something we want?
		if(! command.equalsIgnoreCase("ACTION") && bOriginator) return;

		// Extended Data
		if(command.equalsIgnoreCase("ACTION")) {
			rsh.handle(target, from, "***" + from + " " + data);
			return;
		} else if(command.equalsIgnoreCase("DCC")) {
		} else if(command.equalsIgnoreCase("SED")) {


		// Request/Reply
		} else if(command.equalsIgnoreCase("PING")) {
		} else if(command.equalsIgnoreCase("VERSION")) {
			if(bPrivMsg) {
				mIRC.sendCTCP(from, "VERSION", 
						wtalk.WTalk.getBanner() 
						+ " : " +
						IRC.getBanner()
						+ " : " +
						wtalk.WTalk.getOperatingEnvironment()
							, true);
			}
		} else if(command.equalsIgnoreCase("SOURCE")) {
			if(bPrivMsg) {
				mIRC.sendCTCP(from, "SOURCE",
						wtalk.WTalk.getSourceLocation(),
						true);
			}
		} else if(command.equalsIgnoreCase("USERINFO")) {
		} else if(command.equalsIgnoreCase("CLIENTINFO")) {
		} else if(command.equalsIgnoreCase("ERRMSG")) {
		} else if(command.equalsIgnoreCase("PING")) {
		} else if(command.equalsIgnoreCase("TIME")) {
		}

		// Pass it back to the client?
		// Reverse from and target to make show in the window
		rsh.handle(target, from, "CTCP " + command
				+ " from " + from + ": " + data);	
	}

	public void run(){
		String s;
		String target, from, data;
		IRCMessage message=null;

		if(in==null) return;

		while(true){

			try{
				s = in.readLine();
			} catch(IOException e){ return; }


			if(s==null) break;

			message = new IRCMessage(s);
			target = message.getReceiver();
			from = message.getSender();
			data = message.getText();

			if(message.isServerMsg() ) {

				if(message.getCommand().equals("PING")){
					mIRC.sendPong(from, data);
				} else
					rsh.handle(target, from, data);
				// System.out.println("debug: "+ s);
			} else if(message.isCTCP()) {
				CTCPHandle(message.getCTCPCommand(),
						target, from, data, false, 
						message.getCommand().equalsIgnoreCase("PRIVMSG"));
			} else if(message.isAutomatedMsg()) {
				if ( rah != null) {
					rah.handle(target, target, data);
				}
			} else if(message.getCommand().equals("QUIT")){
				rsh.handle(from, from, message.toString());
			} else if(message.isStatusMsg()
				|| (from==null) || (from.length()==0) ){

				if(target!=null && target.equalsIgnoreCase(
					parent.getName() ))
					target=null;

				if ( (from==null) || (from.length()==0) )
					target=null;

				rsh.handle(target, from, message.toString());
			
			}else if(target != null && target.length() > 0) {

				if( !(target.charAt(0)=='&' || 
					target.charAt(0)=='#')) {
					rph.handle(target, from, data);
				} else {
					rmh.handle(target, from, data);
				}
			}

			
		}

		in=null;

	}
}

